import os
import psycopg2
import json
import logging
from datetime import datetime

# Configure logging
logging.basicConfig(
    level=logging.INFO,
    format='%(asctime)s - %(name)s - %(levelname)s - %(message)s'
)
logger = logging.getLogger('ploughshares')

def get_db_connection():
    host = os.environ.get('POSTGRES_HOST', 'db')
    port = os.environ.get('POSTGRES_PORT', '5432')
    dbname = os.environ.get('POSTGRES_DB', 'ploughshares')
    user = os.environ.get('POSTGRES_USER', 'ploughshares')
    password = os.environ.get('POSTGRES_PASSWORD', 'ploughshares_password')
    
    conn = psycopg2.connect(
        host=host,
        port=port,
        dbname=dbname,
        user=user,
        password=password
    )
    conn.autocommit = True
    return conn

def populate_test_data():
    conn = get_db_connection()
    cursor = conn.cursor()
    
    # Load sample data from data.json
    try:
        with open('data.json', 'r') as f:
            data_json = json.load(f)
            
        # Extract the transactions array
        if 'transactions' not in data_json:
            logger.error("'transactions' key not found in data.json")
            return
            
        data = data_json['transactions']
    except FileNotFoundError:
        logger.error("data.json file not found")
        return
    except json.JSONDecodeError:
        logger.error("data.json is not valid JSON")
        return
    
    # Insert sample transactions
    for item in data:
        try:
            cursor.execute('''
                INSERT INTO transactions (
                    transaction_type, company_division, address_1, address_2,
                    city, province, region, postal_code, is_primary, source_date, source_description,
                    grant_type, description, amount, recipient, commodity_class, contract_number, comments
                ) VALUES (
                    %s, %s, %s, %s,
                    %s, %s, %s, %s, %s, %s, %s,
                    %s, %s, %s, %s, %s, %s, %s
                )
            ''', (
                item.get('transaction_type', ''),
                item.get('company_division', ''),
                item.get('address_1', ''),
                item.get('address_2', ''),
                item.get('city', ''),
                item.get('province', ''),
                item.get('region', ''),
                item.get('postal_code', ''),
                item.get('is_primary', False),
                datetime.strptime(item.get('source_date', '2023-01-01'), '%Y-%m-%d').date() if item.get('source_date') else None,
                item.get('source_description', ''),
                item.get('grant_type', ''),
                item.get('description', ''),
                float(item.get('amount', 0)),
                item.get('recipient', ''),
                item.get('commodity_class', ''),
                item.get('contract_number', ''),
                item.get('comments', '')
            ))
            logger.info(f"Added transaction: {item.get('company_division', 'Unknown')} - {item.get('recipient', 'Unknown')}")
        except Exception as e:
            logger.error(f"Error inserting transaction: {e}")
    
    cursor.close()
    conn.close()
    
    logger.info("Database populated with test data successfully.")

if __name__ == "__main__":
    populate_test_data() 